@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GINKGO_BENCHMARK_UTILS_GENERATOR_HPP
 #define GINKGO_BENCHMARK_UTILS_GENERATOR_HPP
@@ -67,7 +39,7 @@ struct DefaultSystemGenerator {
             throw std::runtime_error(
                 "No known way to generate matrix data found.");
         }
-        data.ensure_row_major_order();
+        data.sort_row_major();
         return data;
     }
 
@@ -209,7 +181,7 @@ struct DistributedDefaultSystemGenerator {
             throw std::runtime_error(
                 "No known way to generate matrix data found.");
         }
-        data.ensure_row_major_order();
+        data.sort_row_major();
         return data;
     }
 
@@ -251,9 +223,11 @@ struct DistributedDefaultSystemGenerator {
         const gko::matrix_data<value_type, index_type>& data,
         json* spmv_case = nullptr) const
     {
-        auto part = gko::experimental::distributed::
-            Partition<itype, global_itype>::build_from_global_size_uniform(
-                exec, comm.size(), static_cast<global_itype>(data.size[0]));
+        auto part = gko::share(
+            gko::experimental::distributed::Partition<itype, global_itype>::
+                build_from_global_size_uniform(
+                    exec, comm.size(),
+                    static_cast<global_itype>(data.size[0])));
         auto formats = split(format_name, '-');
         if (formats.size() != 2) {
             throw std::runtime_error{"Invalid distributed format specifier " +
