@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/base/perturbation.hpp>
 
@@ -95,6 +67,72 @@ Perturbation<ValueType>::Perturbation(Perturbation&& other)
 }
 
 
+template <typename ValueType>
+Perturbation<ValueType>::Perturbation(std::shared_ptr<const Executor> exec)
+    : EnableLinOp<Perturbation>(std::move(exec))
+{}
+
+
+template <typename ValueType>
+Perturbation<ValueType>::Perturbation(std::shared_ptr<const LinOp> scalar,
+                                      std::shared_ptr<const LinOp> basis)
+    : Perturbation(std::move(scalar),
+                   // basis can not be std::move(basis). Otherwise, Program
+                   // deletes basis before applying conjugate transpose
+                   basis,
+                   std::move((as<gko::Transposable>(basis))->conj_transpose()))
+{}
+
+
+template <typename ValueType>
+Perturbation<ValueType>::Perturbation(std::shared_ptr<const LinOp> scalar,
+                                      std::shared_ptr<const LinOp> basis,
+                                      std::shared_ptr<const LinOp> projector)
+    : EnableLinOp<Perturbation>(basis->get_executor(),
+                                gko::dim<2>{basis->get_size()[0]}),
+      scalar_{std::move(scalar)},
+      basis_{std::move(basis)},
+      projector_{std::move(projector)}
+{
+    this->validate_perturbation();
+}
+
+
+template <typename ValueType>
+std::unique_ptr<Perturbation<ValueType>> Perturbation<ValueType>::create(
+    std::shared_ptr<const Executor> exec)
+{
+    return std::unique_ptr<Perturbation>{new Perturbation{exec}};
+}
+
+
+template <typename ValueType>
+std::unique_ptr<Perturbation<ValueType>> Perturbation<ValueType>::create(
+    std::shared_ptr<const LinOp> scalar, std::shared_ptr<const LinOp> basis)
+{
+    return std::unique_ptr<Perturbation>{new Perturbation{scalar, basis}};
+}
+
+
+template <typename ValueType>
+std::unique_ptr<Perturbation<ValueType>> Perturbation<ValueType>::create(
+    std::shared_ptr<const LinOp> scalar, std::shared_ptr<const LinOp> basis,
+    std::shared_ptr<const LinOp> projector)
+{
+    return std::unique_ptr<Perturbation>{
+        new Perturbation{scalar, basis, projector}};
+}
+
+
+template <typename ValueType>
+void Perturbation<ValueType>::validate_perturbation()
+{
+    GKO_ASSERT_CONFORMANT(basis_, projector_);
+    GKO_ASSERT_CONFORMANT(projector_, basis_);
+    GKO_ASSERT_EQUAL_DIMENSIONS(scalar_, dim<2>(1, 1));
+}
+
+
 template <typename ValueType>
 void Perturbation<ValueType>::apply_impl(const LinOp* b, LinOp* x) const
 {
