@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/matrix/diagonal.hpp>
 
@@ -355,6 +327,53 @@ Diagonal<ValueType>::compute_absolute() const
 }
 
 
+template <typename ValueType>
+Diagonal<ValueType>::Diagonal(std::shared_ptr<const Executor> exec,
+                              size_type size)
+    : EnableLinOp<Diagonal>(exec, dim<2>{size}), values_(exec, size)
+{}
+
+
+template <typename ValueType>
+Diagonal<ValueType>::Diagonal(std::shared_ptr<const Executor> exec,
+                              const size_type size, array<value_type> values)
+    : EnableLinOp<Diagonal>(exec, dim<2>(size)),
+      values_{exec, std::move(values)}
+{
+    GKO_ENSURE_IN_BOUNDS(size - 1, values_.get_size());
+}
+
+
+template <typename ValueType>
+std::unique_ptr<Diagonal<ValueType>> Diagonal<ValueType>::create(
+    std::shared_ptr<const Executor> exec, size_type size)
+{
+    return std::unique_ptr<Diagonal>{new Diagonal{exec, size}};
+}
+
+
+template <typename ValueType>
+std::unique_ptr<Diagonal<ValueType>> Diagonal<ValueType>::create(
+    std::shared_ptr<const Executor> exec, const size_type size,
+    array<value_type> values)
+{
+    return std::unique_ptr<Diagonal>{
+        new Diagonal{exec, size, std::move(values)}};
+}
+
+
+template <typename ValueType>
+std::unique_ptr<const Diagonal<ValueType>> Diagonal<ValueType>::create_const(
+    std::shared_ptr<const Executor> exec, size_type size,
+    gko::detail::const_array_view<ValueType>&& values)
+{
+    // cast const-ness away, but return a const object afterwards,
+    // so we can ensure that no modifications take place.
+    return std::unique_ptr<const Diagonal>{new Diagonal{
+        exec, size, gko::detail::array_const_cast(std::move(values))}};
+}
+
+
 #define GKO_DECLARE_DIAGONAL_MATRIX(value_type) class Diagonal<value_type>
 GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DIAGONAL_MATRIX);
 
