@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/matrix/fbcsr.hpp>
 
@@ -417,6 +389,97 @@ Fbcsr<ValueType, IndexType>::compute_absolute() const
 }
 
 
+template <typename ValueType, typename IndexType>
+std::unique_ptr<Fbcsr<ValueType, IndexType>>
+Fbcsr<ValueType, IndexType>::create(std::shared_ptr<const Executor> exec,
+                                    int block_size)
+{
+    return std::unique_ptr<Fbcsr>{new Fbcsr{exec, block_size}};
+}
+
+
+template <typename ValueType, typename IndexType>
+std::unique_ptr<Fbcsr<ValueType, IndexType>>
+Fbcsr<ValueType, IndexType>::create(std::shared_ptr<const Executor> exec,
+                                    const dim<2>& size, size_type num_nonzeros,
+                                    int block_size)
+{
+    return std::unique_ptr<Fbcsr>{
+        new Fbcsr{exec, size, num_nonzeros, block_size}};
+}
+
+
+template <typename ValueType, typename IndexType>
+std::unique_ptr<Fbcsr<ValueType, IndexType>>
+Fbcsr<ValueType, IndexType>::create(std::shared_ptr<const Executor> exec,
+                                    const dim<2>& size, int block_size,
+                                    array<value_type> values,
+                                    array<index_type> col_idxs,
+                                    array<index_type> row_ptrs)
+{
+    return std::unique_ptr<Fbcsr>{
+        new Fbcsr{exec, size, block_size, std::move(values),
+                  std::move(col_idxs), std::move(row_ptrs)}};
+}
+
+
+template <typename ValueType, typename IndexType>
+std::unique_ptr<const Fbcsr<ValueType, IndexType>>
+Fbcsr<ValueType, IndexType>::create_const(
+    std::shared_ptr<const Executor> exec, const dim<2>& size, int blocksize,
+    gko::detail::const_array_view<ValueType>&& values,
+    gko::detail::const_array_view<IndexType>&& col_idxs,
+    gko::detail::const_array_view<IndexType>&& row_ptrs)
+{
+    // cast const-ness away, but return a const object afterwards,
+    // so we can ensure that no modifications take place.
+    return std::unique_ptr<const Fbcsr>{new Fbcsr{
+        exec, size, blocksize, gko::detail::array_const_cast(std::move(values)),
+        gko::detail::array_const_cast(std::move(col_idxs)),
+        gko::detail::array_const_cast(std::move(row_ptrs))}};
+}
+
+
+template <typename ValueType, typename IndexType>
+Fbcsr<ValueType, IndexType>::Fbcsr(std::shared_ptr<const Executor> exec,
+                                   int block_size)
+    : Fbcsr(std::move(exec), dim<2>{}, {}, block_size)
+{}
+
+
+template <typename ValueType, typename IndexType>
+Fbcsr<ValueType, IndexType>::Fbcsr(std::shared_ptr<const Executor> exec,
+                                   const dim<2>& size, size_type num_nonzeros,
+                                   int block_size)
+    : EnableLinOp<Fbcsr>(exec, size),
+      bs_{block_size},
+      values_(exec, num_nonzeros),
+      col_idxs_(exec,
+                detail::get_num_blocks(block_size * block_size, num_nonzeros)),
+      row_ptrs_(exec, detail::get_num_blocks(block_size, size[0]) + 1)
+{
+    GKO_ASSERT_BLOCK_SIZE_CONFORMANT(size[1], bs_);
+    row_ptrs_.fill(0);
+}
+
+
+template <typename ValueType, typename IndexType>
+Fbcsr<ValueType, IndexType>::Fbcsr(std::shared_ptr<const Executor> exec,
+                                   const dim<2>& size, int block_size,
+                                   array<value_type> values,
+                                   array<index_type> col_idxs,
+                                   array<index_type> row_ptrs)
+    : EnableLinOp<Fbcsr>(exec, size),
+      bs_{block_size},
+      values_{exec, std::move(values)},
+      col_idxs_{exec, std::move(col_idxs)},
+      row_ptrs_{exec, std::move(row_ptrs)}
+{
+    GKO_ASSERT_EQ(values_.get_size(), col_idxs_.get_size() * bs_ * bs_);
+    GKO_ASSERT_EQ(this->get_size()[0] / bs_ + 1, row_ptrs_.get_size());
+}
+
+
 #define GKO_DECLARE_FBCSR_MATRIX(ValueType, IndexType) \
     class Fbcsr<ValueType, IndexType>
 GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_FBCSR_MATRIX);
