@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/base/array.hpp>
 
@@ -43,6 +15,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/base/executor.hpp>
 
 
+#include "core/base/array_access.hpp"
 #include "core/test/utils.hpp"
 
 
@@ -61,7 +34,7 @@ protected:
     static void assert_equal_to_original_x(gko::array<T>& a,
                                            bool check_zero = true)
     {
-        ASSERT_EQ(a.get_num_elems(), 2);
+        ASSERT_EQ(a.get_size(), 2);
         if (check_zero) EXPECT_EQ(a.get_data()[0], T{5});
         EXPECT_EQ(a.get_data()[1], T{2});
         if (check_zero) EXPECT_EQ(a.get_const_data()[0], T{5});
@@ -80,7 +53,7 @@ TYPED_TEST(Array, CanBeCreatedWithoutAnExecutor)
     gko::array<TypeParam> a;
 
     ASSERT_EQ(a.get_executor(), nullptr);
-    ASSERT_EQ(a.get_num_elems(), 0);
+    ASSERT_EQ(a.get_size(), 0);
 }
 
 
@@ -88,7 +61,7 @@ TYPED_TEST(Array, CanBeEmpty)
 {
     gko::array<TypeParam> a(this->exec);
 
-    ASSERT_EQ(a.get_num_elems(), 0);
+    ASSERT_EQ(a.get_size(), 0);
 }
 
 
@@ -106,7 +79,7 @@ TYPED_TEST(Array, CanBeCreatedFromExistingData)
     gko::array<TypeParam> a{this->exec, 3, new TypeParam[3],
                             std::default_delete<TypeParam[]>{}};
 
-    EXPECT_EQ(a.get_num_elems(), 3);
+    EXPECT_EQ(a.get_size(), 3);
 }
 
 
@@ -115,7 +88,7 @@ TYPED_TEST(Array, CanBeCreatedFromDataOnExecutor)
     gko::array<TypeParam> a{this->exec, 3,
                             this->exec->template alloc<TypeParam>(3)};
 
-    EXPECT_EQ(a.get_num_elems(), 3);
+    EXPECT_EQ(a.get_size(), 3);
 }
 
 
@@ -142,7 +115,7 @@ TYPED_TEST(Array, CanBeCreatedFromInitializerList)
 }
 
 
-TYPED_TEST(Array, KnowsItsSize) { ASSERT_EQ(this->x.get_num_elems(), 2); }
+TYPED_TEST(Array, KnowsItsSize) { ASSERT_EQ(this->x.get_size(), 2); }
 
 
 TYPED_TEST(Array, ReturnsValidDataPtr)
@@ -205,7 +178,7 @@ TYPED_TEST(Array, MoveConstructedFromArrayExecutorlessIsEmpty)
     a = std::move(this->x);
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -214,7 +187,7 @@ TYPED_TEST(Array, MoveConstructedFromArraySameExecutorIsEmpty)
     gko::array<TypeParam> a{this->exec, std::move(this->x)};
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -224,7 +197,7 @@ TYPED_TEST(Array, MoveConstructedFromArrayDifferentExecutorIsEmpty)
                             std::move(this->x)};
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -258,7 +231,7 @@ TYPED_TEST(Array, CanBeCopiedFromExecutorlessArray)
     this->x = a;
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -291,7 +264,7 @@ TYPED_TEST(Array, CanBeMovedFromExecutorlessArray)
     this->x = std::move(a);
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -302,7 +275,7 @@ TYPED_TEST(Array, MovedFromArrayExecutorlessIsEmpty)
     a = std::move(this->x);
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -313,7 +286,7 @@ TYPED_TEST(Array, MovedFromArraySameExecutorIsEmpty)
     a = std::move(this->x);
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
 }
 
 
@@ -324,7 +297,44 @@ TYPED_TEST(Array, MovedFromArrayDifferentExecutorIsEmpty)
     a = std::move(this->x);
 
     ASSERT_EQ(this->x.get_executor(), this->exec);
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
+}
+
+
+TYPED_TEST(Array, CanGetElement)
+{
+    gko::array<TypeParam> a{this->exec, {2, 4}};
+
+    ASSERT_EQ(get_element(a, 0), TypeParam{2});
+    ASSERT_EQ(get_element(a, 1), TypeParam{4});
+}
+
+
+TYPED_TEST(Array, CanSetElement)
+{
+    gko::array<TypeParam> a{this->exec, {2, 4}};
+
+    set_element(a, 1, TypeParam{0});
+
+    ASSERT_EQ(get_element(a, 1), TypeParam{0});
+}
+
+
+TYPED_TEST(Array, GetElementThrowsOutOfBounds)
+{
+    gko::array<TypeParam> a{this->exec, {2, 4}};
+
+    ASSERT_THROW(get_element(a, 2), gko::OutOfBoundsError);
+    // TODO2.0 add bounds check test for negative indices
+}
+
+
+TYPED_TEST(Array, SetElementThrowsOutOfBounds)
+{
+    gko::array<TypeParam> a{this->exec, {2, 4}};
+
+    ASSERT_THROW(set_element(a, 2, TypeParam{0}), gko::OutOfBoundsError);
+    // TODO2.0 add bounds check test for negative indices
 }
 
 
@@ -381,7 +391,7 @@ TYPED_TEST(Array, CanCreateTemporaryOutputCloneOnDifferentExecutors)
         tmp_clone->get_data()[1] = 5;
 
         // there is no reliable way to check the memory is uninitialized
-        ASSERT_EQ(tmp_clone->get_num_elems(), this->x.get_num_elems());
+        ASSERT_EQ(tmp_clone->get_size(), this->x.get_size());
         ASSERT_EQ(tmp_clone->get_executor(), other);
         ASSERT_EQ(this->x.get_executor(), this->exec);
         ASSERT_EQ(this->x.get_data()[0], TypeParam{5});
@@ -396,7 +406,7 @@ TYPED_TEST(Array, CanBeCleared)
 {
     this->x.clear();
 
-    ASSERT_EQ(this->x.get_num_elems(), 0);
+    ASSERT_EQ(this->x.get_size(), 0);
     ASSERT_EQ(this->x.get_data(), nullptr);
     ASSERT_EQ(this->x.get_const_data(), nullptr);
 }
@@ -422,7 +432,7 @@ TYPED_TEST(Array, ViewCannotBeResized)
     auto view = gko::make_array_view(this->exec, 3, data);
 
     EXPECT_THROW(view.resize_and_reset(1), gko::NotSupported);
-    EXPECT_EQ(view.get_num_elems(), 3);
+    EXPECT_EQ(view.get_size(), 3);
     ASSERT_EQ(view.get_data()[0], TypeParam{1});
 }
 
@@ -451,7 +461,7 @@ TYPED_TEST(Array, CustomDeleterCannotBeResized)
         this->exec, 3, data, my_null_deleter<TypeParam[]>{});
 
     EXPECT_THROW(view_custom_deleter.resize_and_reset(1), gko::NotSupported);
-    EXPECT_EQ(view_custom_deleter.get_num_elems(), 3);
+    EXPECT_EQ(view_custom_deleter.get_size(), 3);
     ASSERT_EQ(view_custom_deleter.get_data()[0], TypeParam{1});
 }
 
@@ -487,7 +497,7 @@ TYPED_TEST(Array, ViewModifiesOriginalData)
     EXPECT_EQ(data[0], TypeParam{5});
     EXPECT_EQ(data[1], TypeParam{4});
     EXPECT_EQ(data[2], TypeParam{2});
-    ASSERT_EQ(view.get_num_elems(), 3);
+    ASSERT_EQ(view.get_size(), 3);
 }
 
 
@@ -502,9 +512,9 @@ TYPED_TEST(Array, CopyArrayToArray)
     EXPECT_EQ(array.get_data()[1], TypeParam{4});
     EXPECT_EQ(array.get_data()[2], TypeParam{2});
     EXPECT_EQ(array.get_data()[3], TypeParam{1});
-    EXPECT_EQ(array.get_num_elems(), 4);
+    EXPECT_EQ(array.get_size(), 4);
     EXPECT_NE(array.get_data(), array2.get_data());
-    ASSERT_EQ(array2.get_num_elems(), 4);
+    ASSERT_EQ(array2.get_size(), 4);
 }
 
 
@@ -523,8 +533,8 @@ TYPED_TEST(Array, CopyViewToView)
     EXPECT_EQ(data[0], TypeParam{5});
     EXPECT_EQ(data[1], TypeParam{4});
     EXPECT_EQ(data[2], TypeParam{2});
-    EXPECT_EQ(view.get_num_elems(), 3);
-    EXPECT_EQ(view2.get_num_elems(), 3);
+    EXPECT_EQ(view.get_size(), 3);
+    EXPECT_EQ(view2.get_size(), 3);
     EXPECT_EQ(view2.get_data()[0], TypeParam{2});
     ASSERT_THROW(view2 = view_size4, gko::OutOfBoundsError);
 }
@@ -543,15 +553,15 @@ TYPED_TEST(Array, CopyViewToArray)
     EXPECT_EQ(array.get_data()[1], TypeParam{2});
     EXPECT_EQ(array.get_data()[2], TypeParam{3});
     EXPECT_EQ(array.get_data()[3], TypeParam{4});
-    EXPECT_EQ(array.get_num_elems(), 4);
-    ASSERT_EQ(view.get_num_elems(), 4);
+    EXPECT_EQ(array.get_size(), 4);
+    ASSERT_EQ(view.get_size(), 4);
 }
 
 
 TYPED_TEST(Array, CopyArrayToView)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::make_array_view(this->exec, 3, data);
+    auto view = gko::make_array_view(this->exec, 2, data);
     gko::array<TypeParam> array_size2(this->exec, {5, 4});
     gko::array<TypeParam> array_size4(this->exec, {5, 4, 2, 1});
 
@@ -559,13 +569,49 @@ TYPED_TEST(Array, CopyArrayToView)
 
     EXPECT_EQ(data[0], TypeParam{5});
     EXPECT_EQ(data[1], TypeParam{4});
-    EXPECT_EQ(data[2], TypeParam{3});
-    EXPECT_EQ(view.get_num_elems(), 3);
-    EXPECT_EQ(array_size2.get_num_elems(), 2);
+    EXPECT_EQ(view.get_size(), 2);
+    EXPECT_EQ(array_size2.get_size(), 2);
     ASSERT_THROW(view = array_size4, gko::OutOfBoundsError);
 }
 
 
+TYPED_TEST(Array, CopyConstViewToArray)
+{
+    TypeParam data[] = {1, 2, 3, 4};
+    auto const_view = gko::make_const_array_view(this->exec, 4, data);
+    gko::array<TypeParam> array(this->exec, {5, 4, 2});
+
+    array = const_view;
+    data[1] = 7;
+
+    EXPECT_EQ(array.get_data()[0], TypeParam{1});
+    EXPECT_EQ(array.get_data()[1], TypeParam{2});
+    EXPECT_EQ(array.get_data()[2], TypeParam{3});
+    EXPECT_EQ(array.get_data()[3], TypeParam{4});
+    EXPECT_EQ(array.get_size(), 4);
+    ASSERT_EQ(const_view.get_size(), 4);
+}
+
+
+TYPED_TEST(Array, CopyConstViewToView)
+{
+    TypeParam data1[] = {1, 2, 3, 4};
+    TypeParam data2[] = {5, 4, 2};
+    auto view = gko::make_array_view(this->exec, 3, data2);
+    auto const_view3 = gko::make_const_array_view(this->exec, 3, data1);
+    auto const_view4 = gko::make_const_array_view(this->exec, 4, data1);
+
+    view = const_view3;
+    data1[1] = 7;
+
+    EXPECT_EQ(view.get_data()[0], TypeParam{1});
+    EXPECT_EQ(view.get_data()[1], TypeParam{2});
+    EXPECT_EQ(view.get_data()[2], TypeParam{3});
+    EXPECT_EQ(view.get_size(), 3);
+    EXPECT_EQ(const_view3.get_size(), 3);
+    ASSERT_THROW(view = const_view4, gko::OutOfBoundsError);
+}
+
 TYPED_TEST(Array, MoveArrayToArray)
 {
     gko::array<TypeParam> array(this->exec, {1, 2, 3});
@@ -579,9 +625,9 @@ TYPED_TEST(Array, MoveArrayToArray)
     EXPECT_EQ(array.get_data()[1], TypeParam{4});
     EXPECT_EQ(array.get_data()[2], TypeParam{2});
     EXPECT_EQ(array.get_data()[3], TypeParam{1});
-    EXPECT_EQ(array.get_num_elems(), 4);
+    EXPECT_EQ(array.get_size(), 4);
     EXPECT_EQ(array2.get_data(), nullptr);
-    ASSERT_EQ(array2.get_num_elems(), 0);
+    ASSERT_EQ(array2.get_size(), 0);
 }
 
 
@@ -598,9 +644,9 @@ TYPED_TEST(Array, MoveViewToView)
     EXPECT_EQ(view.get_data()[0], TypeParam{5});
     EXPECT_EQ(view.get_data()[1], TypeParam{4});
     EXPECT_EQ(view.get_data()[2], TypeParam{2});
-    EXPECT_EQ(view.get_num_elems(), 3);
+    EXPECT_EQ(view.get_size(), 3);
     EXPECT_EQ(view2.get_data(), nullptr);
-    EXPECT_EQ(view2.get_num_elems(), 0);
+    EXPECT_EQ(view2.get_size(), 0);
     EXPECT_NE(data, nullptr);
     EXPECT_EQ(data[0], TypeParam{1});
     EXPECT_EQ(data[1], TypeParam{2});
@@ -622,13 +668,13 @@ TYPED_TEST(Array, MoveViewToArray)
     EXPECT_EQ(array.get_data()[1], TypeParam{2});
     EXPECT_EQ(array.get_data()[2], TypeParam{3});
     EXPECT_EQ(array.get_data()[3], TypeParam{4});
-    EXPECT_EQ(array.get_num_elems(), 4);
+    EXPECT_EQ(array.get_size(), 4);
     EXPECT_EQ(data[0], TypeParam{1});
     EXPECT_EQ(data[1], TypeParam{2});
     EXPECT_EQ(data[2], TypeParam{3});
     EXPECT_EQ(data[3], TypeParam{4});
     EXPECT_EQ(view.get_data(), nullptr);
-    ASSERT_EQ(view.get_num_elems(), 0);
+    ASSERT_EQ(view.get_size(), 0);
 }
 
 
@@ -645,27 +691,27 @@ TYPED_TEST(Array, MoveArrayToView)
 
     EXPECT_EQ(view.get_data()[0], TypeParam{5});
     EXPECT_EQ(view.get_data()[1], TypeParam{4});
-    EXPECT_EQ(view.get_num_elems(), 2);
+    EXPECT_EQ(view.get_size(), 2);
     EXPECT_NE(view.get_data(), data);
     EXPECT_EQ(view.get_data(), size2_ptr);
     EXPECT_NO_THROW(view = std::move(array_size4));
     EXPECT_EQ(view.get_data(), size4_ptr);
     EXPECT_EQ(array_size2.get_data(), nullptr);
-    ASSERT_EQ(array_size2.get_num_elems(), 0);
+    ASSERT_EQ(array_size2.get_size(), 0);
 }
 
 
 TYPED_TEST(Array, AsView)
 {
     auto ptr = this->x.get_data();
-    auto size = this->x.get_num_elems();
+    auto size = this->x.get_size();
     auto exec = this->x.get_executor();
     auto view = this->x.as_view();
 
     ASSERT_EQ(ptr, this->x.get_data());
     ASSERT_EQ(ptr, view.get_data());
-    ASSERT_EQ(size, this->x.get_num_elems());
-    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(size, this->x.get_size());
+    ASSERT_EQ(size, view.get_size());
     ASSERT_EQ(exec, this->x.get_executor());
     ASSERT_EQ(exec, view.get_executor());
     ASSERT_TRUE(this->x.is_owning());
@@ -676,14 +722,14 @@ TYPED_TEST(Array, AsView)
 TYPED_TEST(Array, AsConstView)
 {
     auto ptr = this->x.get_data();
-    auto size = this->x.get_num_elems();
+    auto size = this->x.get_size();
     auto exec = this->x.get_executor();
     auto view = this->x.as_const_view();
 
     ASSERT_EQ(ptr, this->x.get_data());
     ASSERT_EQ(ptr, view.get_const_data());
-    ASSERT_EQ(size, this->x.get_num_elems());
-    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(size, this->x.get_size());
+    ASSERT_EQ(size, view.get_size());
     ASSERT_EQ(exec, this->x.get_executor());
     ASSERT_EQ(exec, view.get_executor());
     ASSERT_TRUE(this->x.is_owning());
@@ -694,7 +740,7 @@ TYPED_TEST(Array, AsConstView)
 TYPED_TEST(Array, ArrayConstCastWorksOnView)
 {
     auto ptr = this->x.get_data();
-    auto size = this->x.get_num_elems();
+    auto size = this->x.get_size();
     auto exec = this->x.get_executor();
     auto const_view = this->x.as_const_view();
     auto view = gko::detail::array_const_cast(std::move(const_view));
@@ -702,12 +748,12 @@ TYPED_TEST(Array, ArrayConstCastWorksOnView)
                   "wrong return type");
 
     ASSERT_EQ(nullptr, const_view.get_const_data());
-    ASSERT_EQ(0, const_view.get_num_elems());
+    ASSERT_EQ(0, const_view.get_size());
     ASSERT_EQ(exec, const_view.get_executor());
     ASSERT_EQ(ptr, this->x.get_data());
     ASSERT_EQ(ptr, view.get_const_data());
-    ASSERT_EQ(size, this->x.get_num_elems());
-    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(size, this->x.get_size());
+    ASSERT_EQ(size, view.get_size());
     ASSERT_EQ(exec, this->x.get_executor());
     ASSERT_EQ(exec, view.get_executor());
     ASSERT_TRUE(this->x.is_owning());
