@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/base/mtx_io.hpp>
 
@@ -418,6 +390,22 @@ TEST(MtxReader, ReadsSparseComplexHermitianMtx)
 }
 
 
+TEST(MtxReader, ReadIgnoresExtraCharacters)
+{
+    using tpl = gko::matrix_data<double, gko::int32>::nonzero_type;
+    std::istringstream iss(
+        "%%MatrixMarket matrix array real general ??? whateve\r\n"
+        "1 1\n"
+        "0.1\n");
+
+    auto data = gko::read_raw<double, gko::int32>(iss);
+
+    ASSERT_EQ(data.size, gko::dim<2>(1, 1));
+    auto& v = data.nonzeros;
+    ASSERT_EQ(v[0], tpl(0, 0, 0.1));
+}
+
+
 std::array<gko::uint64, 20> build_binary_complex_data()
 {
     gko::uint64 int_val{};
@@ -670,7 +658,7 @@ TEST(MatrixData, WritesDoubleRealMatrixToMatrixMarketArray)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data);
+    write_raw(oss, data, gko::layout_type::array);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix array real general\n"
@@ -694,7 +682,7 @@ TEST(MatrixData, WritesFloatRealMatrixToMatrixMarketCoordinate)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data, gko::layout_type::coordinate);
+    write_raw(oss, data);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix coordinate real general\n"
@@ -717,7 +705,7 @@ TEST(MatrixData, WritesDoubleRealMatrixToMatrixMarketArrayWith64Index)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data);
+    write_raw(oss, data, gko::layout_type::array);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix array real general\n"
@@ -741,7 +729,7 @@ TEST(MatrixData, WritesFloatRealMatrixToMatrixMarketCoordinateWith64Index)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data, gko::layout_type::coordinate);
+    write_raw(oss, data);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix coordinate real general\n"
@@ -764,7 +752,7 @@ TEST(MatrixData, WritesComplexDoubleMatrixToMatrixMarketArray)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data);
+    write_raw(oss, data, gko::layout_type::array);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix array complex general\n"
@@ -788,7 +776,7 @@ TEST(MatrixData, WritesComplexFloatMatrixToMatrixMarketCoordinate)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data, gko::layout_type::coordinate);
+    write_raw(oss, data);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix coordinate complex general\n"
@@ -811,7 +799,7 @@ TEST(MatrixData, WritesComplexDoubleMatrixToMatrixMarketArrayWith64Index)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data);
+    write_raw(oss, data, gko::layout_type::array);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix array complex general\n"
@@ -835,7 +823,7 @@ TEST(MatrixData, WritesComplexFloatMatrixToMatrixMarketCoordinateWith64Index)
     // clang-format on
     std::ostringstream oss{};
 
-    write_raw(oss, data, gko::layout_type::coordinate);
+    write_raw(oss, data);
 
     ASSERT_EQ(oss.str(),
               "%%MatrixMarket matrix coordinate complex general\n"
