@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/log/profiler_hook.hpp>
 
@@ -138,8 +110,8 @@ TEST(ProfilerHook, LogsPolymorphicObjectLinOp)
 {
     std::vector<std::string> expected{"begin:copy(obj,obj)",
                                       "end:copy(obj,obj)",
-                                      "begin:move(obj,obj)",
-                                      "end:move(obj,obj)",
+                                      "begin:move(obj_copy,obj)",
+                                      "end:move(obj_copy,obj)",
                                       "begin:apply(obj)",
                                       "begin:op",
                                       "end:op",
@@ -160,14 +132,18 @@ TEST(ProfilerHook, LogsPolymorphicObjectLinOp)
     auto logger = gko::log::ProfilerHook::create_custom(
         std::move(hooks.first), std::move(hooks.second));
     auto linop = gko::share(DummyLinOp::create(exec));
+    auto linop_copy = linop->clone();
     auto factory = DummyLinOp::build().on(exec);
     auto scalar = DummyLinOp::create(exec, gko::dim<2>{1, 1});
     logger->set_object_name(linop, "obj");
+    logger->set_object_name(linop_copy, "obj_copy");
     logger->set_object_name(factory, "obj_factory");
     exec->add_logger(logger);
 
     linop->copy_from(linop);
-    linop->move_from(linop);
+    // self move-assignment is potentially illegal for std::vector in pre-C++23,
+    // this would causes the libstdc++ debug mode to abort, so use the copy
+    linop->move_from(linop_copy);
     linop->apply(linop, linop);
     linop->apply(scalar, linop, scalar, linop);
     factory->generate(linop);
