@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/solver/workspace.hpp>
 
@@ -135,7 +107,7 @@ TEST_F(Workspace, AnyArrayInitWorks)
     ASSERT_TRUE(array.template contains<int>());
     ASSERT_FALSE(array.template contains<double>());
     ASSERT_EQ(&array.template get<int>(), &arr);
-    ASSERT_EQ(arr.get_num_elems(), 1);
+    ASSERT_EQ(arr.get_size(), 1);
     ASSERT_EQ(arr.get_executor(), exec);
 }
 
@@ -160,8 +132,8 @@ TEST_F(Workspace, CanCreateArrays)
     auto& arr1 = ws.create_or_get_array<int>(1, 2);
     auto& arr2 = ws.create_or_get_array<double>(0, 3);
 
-    ASSERT_EQ(arr1.get_num_elems(), 2);
-    ASSERT_EQ(arr2.get_num_elems(), 3);
+    ASSERT_EQ(arr1.get_size(), 2);
+    ASSERT_EQ(arr2.get_size(), 3);
     ASSERT_EQ(arr1.get_executor(), exec);
     ASSERT_EQ(arr2.get_executor(), exec);
 }
@@ -177,8 +149,8 @@ TEST_F(Workspace, CanReuseArrays)
     auto& arr1_reuse = ws.create_or_get_array<int>(1, 2);
     auto& arr2_reuse = ws.create_or_get_array<double>(0, 3);
 
-    ASSERT_EQ(arr1.get_num_elems(), 2);
-    ASSERT_EQ(arr2.get_num_elems(), 3);
+    ASSERT_EQ(arr1.get_size(), 2);
+    ASSERT_EQ(arr2.get_size(), 3);
     ASSERT_EQ(arr1.get_executor(), exec);
     ASSERT_EQ(arr2.get_executor(), exec);
     ASSERT_EQ(&arr1, &arr1_reuse);
@@ -196,8 +168,8 @@ TEST_F(Workspace, CanResizeArrays)
     auto& arr1_reuse = ws.create_or_get_array<int>(1, 4);
     auto& arr2_reuse = ws.create_or_get_array<double>(0, 5);
 
-    ASSERT_EQ(arr1.get_num_elems(), 4);
-    ASSERT_EQ(arr2.get_num_elems(), 5);
+    ASSERT_EQ(arr1.get_size(), 4);
+    ASSERT_EQ(arr2.get_size(), 5);
     ASSERT_EQ(arr1.get_executor(), exec);
     ASSERT_EQ(arr2.get_executor(), exec);
     ASSERT_EQ(&arr1, &arr1_reuse);
