@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/test/utils/assertions.hpp"
 
@@ -203,4 +175,50 @@ TEST(BiggestValueType, WithBothComplex)
 }
 
 
+class ArraysNear : public ::testing::Test {
+protected:
+    using ValueArray = gko::array<double>;
+
+    ArraysNear()
+        : exec(gko::ReferenceExecutor::create()),
+          arr1(exec, {1.0, 2.0, 3.0, 4.2}),
+          arr2(exec, {1.0, 2.0, 3.0, 4.2}),
+          arr3(exec, {1.0, 2.0, 3.00001, 4.2}),
+          arr4(exec, {1.0, 2.0, 3.0, 4.3})
+    {}
+
+    std::shared_ptr<const gko::Executor> exec;
+    ValueArray arr1;
+    ValueArray arr2;
+    ValueArray arr3;
+    ValueArray arr4;
+};
+
+
+TEST_F(ArraysNear, SucceedsIfEqual)
+{
+    ASSERT_PRED_FORMAT3(gko::test::assertions::array_near, arr1, arr2, 0.0);
+}
+
+
+TEST_F(ArraysNear, FailsIfDifferent)
+{
+    ASSERT_PRED_FORMAT3(!gko::test::assertions::array_near, arr1, arr4, 0.0);
+}
+
+
+TEST_F(ArraysNear, SucceedsIfClose)
+{
+    ASSERT_PRED_FORMAT3(!gko::test::assertions::array_near, arr1, arr3, 0.0);
+    ASSERT_PRED_FORMAT3(gko::test::assertions::array_near, arr1, arr3, 0.0001);
+}
+
+
+TEST_F(ArraysNear, CanUseShortNotation)
+{
+    GKO_EXPECT_ARRAY_NEAR(arr1, arr2, 0.0);
+    GKO_ASSERT_ARRAY_NEAR(arr1, arr3, 0.001);
+}
+
+
 }  // namespace
