@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "common/unified/base/kernel_launch.hpp"
 
@@ -48,6 +20,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "common/unified/base/kernel_launch_reduction.hpp"
 #include "common/unified/base/kernel_launch_solver.hpp"
+#include "core/base/array_access.hpp"
 #include "core/test/utils.hpp"
 
 
@@ -144,7 +117,7 @@ TEST_F(KernelLaunch, Runs1D)
             static_assert(is_same<decltype(dummy), int64>::value, "dummy");
             d[i] = i;
         },
-        zero_array.get_num_elems(), zero_array.get_data(), move_only_val);
+        zero_array.get_size(), zero_array.get_data(), move_only_val);
 
     GKO_ASSERT_ARRAY_EQ(zero_array, iota_array);
 }
@@ -165,7 +138,7 @@ TEST_F(KernelLaunch, Runs1DArray)
                 d[i] = 0;
             }
         },
-        zero_array.get_num_elems(), zero_array, zero_array.get_const_data(),
+        zero_array.get_size(), zero_array, zero_array.get_const_data(),
         move_only_val);
 
     GKO_ASSERT_ARRAY_EQ(zero_array, iota_array);
@@ -315,7 +288,7 @@ TEST_F(KernelLaunch, Reduction1D)
         int64{}, output.get_data(), size_type{100000}, output, move_only_val);
 
     // 2 * sum i=0...99999 (i+1)
-    EXPECT_EQ(exec->copy_val_to_host(output.get_const_data()), 10000100000LL);
+    EXPECT_EQ(get_element(output, 0), 10000100000LL);
 
     gko::kernels::dpcpp::run_kernel_reduction(
         exec,
@@ -337,7 +310,7 @@ TEST_F(KernelLaunch, Reduction1D)
         int64{}, output.get_data(), size_type{100}, output, move_only_val);
 
     // 2 * sum i=0...99 (i+1)
-    EXPECT_EQ(exec->copy_val_to_host(output.get_const_data()), 10100LL);
+    EXPECT_EQ(get_element(output, 0), 10100LL);
 }
 
 
@@ -367,7 +340,7 @@ TEST_F(KernelLaunch, Reduction2D)
         move_only_val);
 
     // 4 * sum i=0...999 sum j=0...99 of (i+1)*(j+1)
-    EXPECT_EQ(exec->copy_val_to_host(output.get_const_data()), 10110100000LL);
+    EXPECT_EQ(get_element(output, 0), 10110100000LL);
 
     gko::kernels::dpcpp::run_kernel_reduction(
         exec,
@@ -390,7 +363,7 @@ TEST_F(KernelLaunch, Reduction2D)
         int64{}, output.get_data(), gko::dim<2>{10, 10}, output, move_only_val);
 
     // 4 * sum i=0...9 sum j=0...9 of (i+1)*(j+1)
-    ASSERT_EQ(exec->copy_val_to_host(output.get_const_data()), 12100LL);
+    ASSERT_EQ(get_element(output, 0), 12100LL);
 }
 
 
