@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GKO_PUBLIC_CORE_BASE_LIN_OP_HPP_
 #define GKO_PUBLIC_CORE_BASE_LIN_OP_HPP_
@@ -520,6 +492,9 @@ public:
      * In the resulting LinOp, the entry at location `(i,j)` contains the input
      * value `(perm[i],perm[j])`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $P A P^T$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order.
      *
@@ -530,7 +505,7 @@ public:
     {
         return as<Permutable>(this->row_permute(permutation_indices))
             ->column_permute(permutation_indices);
-    };
+    }
 
     /**
      * Returns a LinOp representing the symmetric inverse row and column
@@ -538,6 +513,9 @@ public:
      * In the resulting LinOp, the entry at location `(perm[i],perm[j])`
      * contains the input value `(i,j)`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $P^{-1} A P^{-T}$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order.
      *
@@ -548,13 +526,16 @@ public:
     {
         return as<Permutable>(this->inverse_row_permute(permutation_indices))
             ->inverse_column_permute(permutation_indices);
-    };
+    }
 
     /**
      * Returns a LinOp representing the row permutation of the Permutable
      * object.
      * In the resulting LinOp, the row `i` contains the input row `perm[i]`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $P A$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order.
      *
@@ -569,6 +550,9 @@ public:
      * In the resulting LinOp, the column `i` contains the input column
      * `perm[i]`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $A P^T$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order `perm`.
      *
@@ -582,6 +566,9 @@ public:
      * object.
      * In the resulting LinOp, the row `perm[i]` contains the input row `i`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $P^{-1} A$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order `perm`.
      *
@@ -596,6 +583,9 @@ public:
      * In the resulting LinOp, the column `perm[i]` contains the input column
      * `i`.
      *
+     * From the linear algebra perspective, with $P_{ij} = \delta_{i
+     * \pi(i)}$, this represents the operation $A P^{-T}$.
+     *
      * @param permutation_indices  the array of indices containing the
      *                             permutation order `perm`.
      *
@@ -931,7 +921,7 @@ protected:
  * template parameters to enable a subclass of LinOpFactory.
  *
  * @tparam ConcreteFactory  the concrete factory which is being implemented
- *                          [CRTP parmeter]
+ *                          [CRTP parameter]
  * @tparam ConcreteLinOp  the concrete LinOp type which this factory produces,
  *                        needs to have a constructor which takes a
  *                        const ConcreteFactory *, and an
@@ -949,26 +939,6 @@ using EnableDefaultLinOpFactory =
     EnableDefaultFactory<ConcreteFactory, ConcreteLinOp, ParametersType,
                          PolymorphicBase>;
 
-/**
- * This Macro will generate a new type containing the parameters for the factory
- * `_factory_name`. For more details, see #GKO_ENABLE_LIN_OP_FACTORY().
- * It is required to use this macro **before** calling the
- * macro #GKO_ENABLE_LIN_OP_FACTORY().
- * It is also required to use the same names for all parameters between both
- * macros.
- *
- * @param _parameters_name  name of the parameters member in the class
- * @param _factory_name  name of the generated factory type
- *
- * @ingroup LinOp
- */
-#define GKO_CREATE_FACTORY_PARAMETERS(_parameters_name, _factory_name)  \
-public:                                                                 \
-    class _factory_name;                                                \
-    struct _parameters_name##_type                                      \
-        : public ::gko::enable_parameters_type<_parameters_name##_type, \
-                                               _factory_name>
-
 
 /**
  * This macro will generate a default implementation of a LinOpFactory for the
@@ -1084,134 +1054,6 @@ public:                                                                      \
                   "semi-colon warnings")
 
 
-/**
- * Defines a build method for the factory, simplifying its construction by
- * removing the repetitive typing of factory's name.
- *
- * @param _factory_name  the factory for which to define the method
- *
- * @ingroup LinOp
- */
-#define GKO_ENABLE_BUILD_METHOD(_factory_name)                               \
-    static auto build()->decltype(_factory_name::create())                   \
-    {                                                                        \
-        return _factory_name::create();                                      \
-    }                                                                        \
-    static_assert(true,                                                      \
-                  "This assert is used to counter the false positive extra " \
-                  "semi-colon warnings")
-
-
-#if !(defined(__CUDACC__) || defined(__HIPCC__))
-/**
- * Creates a factory parameter in the factory parameters structure.
- *
- * @param _name  name of the parameter
- * @param __VA_ARGS__  default value of the parameter
- *
- * @see GKO_ENABLE_LIN_OP_FACTORY for more details, and usage example
- *
- * @deprecated Use GKO_FACTORY_PARAMETER_SCALAR or GKO_FACTORY_PARAMETER_VECTOR
- *
- * @ingroup LinOp
- */
-#define GKO_FACTORY_PARAMETER(_name, ...)                                    \
-    mutable _name{__VA_ARGS__};                                              \
-                                                                             \
-    template <typename... Args>                                              \
-    auto with_##_name(Args&&... _value)                                      \
-        const->const std::decay_t<decltype(*this)>&                          \
-    {                                                                        \
-        using type = decltype(this->_name);                                  \
-        this->_name = type{std::forward<Args>(_value)...};                   \
-        return *this;                                                        \
-    }                                                                        \
-    static_assert(true,                                                      \
-                  "This assert is used to counter the false positive extra " \
-                  "semi-colon warnings")
-
-/**
- * Creates a scalar factory parameter in the factory parameters structure.
- *
- * Scalar in this context means that the constructor for this type only takes
- * a single parameter.
- *
- * @param _name  name of the parameter
- * @param _default  default value of the parameter
- *
- * @see GKO_ENABLE_LIN_OP_FACTORY for more details, and usage example
- *
- * @ingroup LinOp
- */
-#define GKO_FACTORY_PARAMETER_SCALAR(_name, _default) \
-    GKO_FACTORY_PARAMETER(_name, _default)
-
-/**
- * Creates a vector factory parameter in the factory parameters structure.
- *
- * Vector in this context means that the constructor for this type takes
- * multiple parameters.
- *
- * @param _name  name of the parameter
- * @param _default  default value of the parameter
- *
- * @see GKO_ENABLE_LIN_OP_FACTORY for more details, and usage example
- *
- * @ingroup LinOp
- */
-#define GKO_FACTORY_PARAMETER_VECTOR(_name, ...) \
-    GKO_FACTORY_PARAMETER(_name, __VA_ARGS__)
-#else  // defined(__CUDACC__) || defined(__HIPCC__)
-// A workaround for the NVCC compiler - parameter pack expansion does not work
-// properly, because while the assignment to a scalar value is translated by
-// cudafe into a C-style cast, the parameter pack expansion is not removed and
-// `Args&&... args` is still kept as a parameter pack.
-#define GKO_FACTORY_PARAMETER(_name, ...)                                    \
-    mutable _name{__VA_ARGS__};                                              \
-                                                                             \
-    template <typename... Args>                                              \
-    auto with_##_name(Args&&... _value)                                      \
-        const->const std::decay_t<decltype(*this)>&                          \
-    {                                                                        \
-        GKO_NOT_IMPLEMENTED;                                                 \
-        return *this;                                                        \
-    }                                                                        \
-    static_assert(true,                                                      \
-                  "This assert is used to counter the false positive extra " \
-                  "semi-colon warnings")
-
-#define GKO_FACTORY_PARAMETER_SCALAR(_name, _default)                        \
-    mutable _name{_default};                                                 \
-                                                                             \
-    template <typename Arg>                                                  \
-    auto with_##_name(Arg&& _value)                                          \
-        const->const std::decay_t<decltype(*this)>&                          \
-    {                                                                        \
-        using type = decltype(this->_name);                                  \
-        this->_name = type{std::forward<Arg>(_value)};                       \
-        return *this;                                                        \
-    }                                                                        \
-    static_assert(true,                                                      \
-                  "This assert is used to counter the false positive extra " \
-                  "semi-colon warnings")
-
-#define GKO_FACTORY_PARAMETER_VECTOR(_name, ...)                             \
-    mutable _name{__VA_ARGS__};                                              \
-                                                                             \
-    template <typename... Args>                                              \
-    auto with_##_name(Args&&... _value)                                      \
-        const->const std::decay_t<decltype(*this)>&                          \
-    {                                                                        \
-        using type = decltype(this->_name);                                  \
-        this->_name = type{std::forward<Args>(_value)...};                   \
-        return *this;                                                        \
-    }                                                                        \
-    static_assert(true,                                                      \
-                  "This assert is used to counter the false positive extra " \
-                  "semi-colon warnings")
-#endif  // defined(__CUDACC__) || defined(__HIPCC__)
-
-
 }  // namespace gko
 
 
