@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GKO_PUBLIC_CORE_DISTRIBUTED_VECTOR_HPP_
 #define GKO_PUBLIC_CORE_DISTRIBUTED_VECTOR_HPP_
@@ -86,11 +58,9 @@ class Partition;
 template <typename ValueType = double>
 class Vector
     : public EnableDistributedLinOp<Vector<ValueType>>,
-      public EnableCreateMethod<Vector<ValueType>>,
       public ConvertibleTo<Vector<next_precision<ValueType>>>,
       public EnableAbsoluteComputation<remove_complex<Vector<ValueType>>>,
       public DistributedBase {
-    friend class EnableCreateMethod<Vector>;
     friend class EnableDistributedPolymorphicObject<Vector, LinOp>;
     friend class Vector<to_complex<ValueType>>;
     friend class Vector<remove_complex<ValueType>>;
@@ -337,7 +307,7 @@ public:
                           array<char>& tmp) const;
 
     /**
-     * Computes the square of the column-wise Euclidian ($L^2$) norm of this
+     * Computes the square of the column-wise Euclidean ($L^2$) norm of this
      * (multi-)vector using a global reduction.
      *
      * @param result  a Dense row vector, used to store the norm
@@ -347,7 +317,7 @@ public:
     void compute_squared_norm2(ptr_param<LinOp> result) const;
 
     /**
-     * Computes the square of the column-wise Euclidian ($L^2$) norm of this
+     * Computes the square of the column-wise Euclidean ($L^2$) norm of this
      * (multi-)vector using a global reduction.
      *
      * @param result  a Dense row vector, used to store the norm
@@ -360,7 +330,7 @@ public:
     void compute_squared_norm2(ptr_param<LinOp> result, array<char>& tmp) const;
 
     /**
-     * Computes the Euclidian (L^2) norm of this (multi-)vector using a global
+     * Computes the Euclidean (L^2) norm of this (multi-)vector using a global
      * reduction.
      *
      * @param result  a Dense row matrix, used to store the norm
@@ -370,7 +340,7 @@ public:
     void compute_norm2(ptr_param<LinOp> result) const;
 
     /**
-     * Computes the Euclidian (L^2) norm of this (multi-)vector using a global
+     * Computes the Euclidean (L^2) norm of this (multi-)vector using a global
      * reduction.
      *
      * @param result  a Dense row matrix, used to store the norm
@@ -404,6 +374,29 @@ public:
      */
     void compute_norm1(ptr_param<LinOp> result, array<char>& tmp) const;
 
+    /**
+     * Computes the column-wise mean of this (multi-)vector using a global
+     * reduction.
+     *
+     * @param result  a Dense row matrix, used to store the mean
+     *                (the number of columns in result must match the number
+     *                of columns of this)
+     */
+    void compute_mean(ptr_param<LinOp> result) const;
+
+    /**
+     * Computes the column-wise arithmetic mean of this (multi-)vector using a
+     * global reduction.
+     *
+     * @param result  a Dense row matrix, used to store the mean
+     *                (the number of columns in result must match the number
+     *                of columns of this)
+     * @param tmp  the temporary storage to use for partial sums during the
+     *             reduction computation. It may be resized and/or reset to the
+     *             correct executor.
+     */
+    void compute_mean(ptr_param<LinOp> result, array<char>& tmp) const;
+
     /**
      * Returns a single element of the multi-vector.
      *
@@ -481,36 +474,6 @@ public:
 
     size_type get_stride() const noexcept { return local_.get_stride(); }
 
-    /**
-     * Creates a constant (immutable) distributed Vector from a constant local
-     * vector.
-     *
-     * @param exec  Executor associated with this vector
-     * @param comm  Communicator associated with this vector
-     * @param global_size  The global size of the vector
-     * @param local_vector  The underlying local vector, of which a view is
-     *                      created
-     */
-    static std::unique_ptr<const Vector> create_const(
-        std::shared_ptr<const Executor> exec, mpi::communicator comm,
-        dim<2> global_size,
-        std::unique_ptr<const local_vector_type> local_vector);
-
-    /**
-     * Creates a constant (immutable) distributed Vector from a constant local
-     * vector. The global size will be deduced from the local sizes, which will
-     * incur a collective communication.
-     *
-     * @param exec  Executor associated with this vector
-     * @param comm  Communicator associated with this vector
-     * @param local_vector  The underlying local vector, of which a view is
-     *                      created
-     */
-    static std::unique_ptr<const Vector> create_const(
-        std::shared_ptr<const Executor> exec, mpi::communicator comm,
-        std::unique_ptr<const local_vector_type> local_vector);
-
-protected:
     /**
      * Creates an empty distributed vector with a specified size
      *
@@ -519,9 +482,13 @@ protected:
      * @param global_size  Global size of the vector
      * @param local_size  Processor-local size of the vector
      * @param stride  Stride of the local vector.
+     *
+     * @return A smart pointer to the newly created vector.
      */
-    Vector(std::shared_ptr<const Executor> exec, mpi::communicator comm,
-           dim<2> global_size, dim<2> local_size, size_type stride);
+    static std::unique_ptr<Vector> create(std::shared_ptr<const Executor> exec,
+                                          mpi::communicator comm,
+                                          dim<2> global_size, dim<2> local_size,
+                                          size_type stride);
 
     /**
      * Creates an empty distributed vector with a specified size
@@ -531,10 +498,13 @@ protected:
      * @param global_size  Global size of the vector
      * @param local_size  Processor-local size of the vector, uses local_size[1]
      *                    as the stride
+     *
+     * @return A smart pointer to the newly created vector.
      */
-    explicit Vector(std::shared_ptr<const Executor> exec,
-                    mpi::communicator comm, dim<2> global_size = {},
-                    dim<2> local_size = {});
+    static std::unique_ptr<Vector> create(std::shared_ptr<const Executor> exec,
+                                          mpi::communicator comm,
+                                          dim<2> global_size = {},
+                                          dim<2> local_size = {});
 
     /**
      * Creates a distributed vector from local vectors with a specified size.
@@ -550,9 +520,12 @@ protected:
      * @param global_size  The global size of the vector
      * @param local_vector  The underlying local vector, the data will be moved
      *                      into this
+     *
+     * @return A smart pointer to the newly created vector.
      */
-    Vector(std::shared_ptr<const Executor> exec, mpi::communicator comm,
-           dim<2> global_size, std::unique_ptr<local_vector_type> local_vector);
+    static std::unique_ptr<Vector> create(
+        std::shared_ptr<const Executor> exec, mpi::communicator comm,
+        dim<2> global_size, std::unique_ptr<local_vector_type> local_vector);
 
     /**
      * Creates a distributed vector from local vectors. The global size will
@@ -569,7 +542,57 @@ protected:
      * @param comm  Communicator associated with this vector
      * @param local_vector  The underlying local vector, the data will be moved
      *                      into this.
+     *
+     * @return A smart pointer to the newly created vector.
      */
+    static std::unique_ptr<Vector> create(
+        std::shared_ptr<const Executor> exec, mpi::communicator comm,
+        std::unique_ptr<local_vector_type> local_vector);
+
+    /**
+     * Creates a constant (immutable) distributed Vector from a constant local
+     * vector.
+     *
+     * @param exec  Executor associated with this vector
+     * @param comm  Communicator associated with this vector
+     * @param global_size  The global size of the vector
+     * @param local_vector  The underlying local vector, of which a view is
+     *                      created
+     *
+     * @return A smart pointer to the newly created vector.
+     */
+    static std::unique_ptr<const Vector> create_const(
+        std::shared_ptr<const Executor> exec, mpi::communicator comm,
+        dim<2> global_size,
+        std::unique_ptr<const local_vector_type> local_vector);
+
+    /**
+     * Creates a constant (immutable) distributed Vector from a constant local
+     * vector. The global size will be deduced from the local sizes, which will
+     * incur a collective communication.
+     *
+     * @param exec  Executor associated with this vector
+     * @param comm  Communicator associated with this vector
+     * @param local_vector  The underlying local vector, of which a view is
+     *                      created
+     *
+     * @return A smart pointer to the newly created vector.
+     */
+    static std::unique_ptr<const Vector> create_const(
+        std::shared_ptr<const Executor> exec, mpi::communicator comm,
+        std::unique_ptr<const local_vector_type> local_vector);
+
+protected:
+    Vector(std::shared_ptr<const Executor> exec, mpi::communicator comm,
+           dim<2> global_size, dim<2> local_size, size_type stride);
+
+    explicit Vector(std::shared_ptr<const Executor> exec,
+                    mpi::communicator comm, dim<2> global_size = {},
+                    dim<2> local_size = {});
+
+    Vector(std::shared_ptr<const Executor> exec, mpi::communicator comm,
+           dim<2> global_size, std::unique_ptr<local_vector_type> local_vector);
+
     Vector(std::shared_ptr<const Executor> exec, mpi::communicator comm,
            std::unique_ptr<local_vector_type> local_vector);
 
