@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GKO_PUBLIC_CORE_MATRIX_DIAGONAL_HPP_
 #define GKO_PUBLIC_CORE_MATRIX_DIAGONAL_HPP_
@@ -67,7 +39,6 @@ class Dense;
 template <typename ValueType = default_precision>
 class Diagonal
     : public EnableLinOp<Diagonal<ValueType>>,
-      public EnableCreateMethod<Diagonal<ValueType>>,
       public ConvertibleTo<Csr<ValueType, int32>>,
       public ConvertibleTo<Csr<ValueType, int64>>,
       public ConvertibleTo<Diagonal<next_precision<ValueType>>>,
@@ -78,7 +49,6 @@ class Diagonal
       public ReadableFromMatrixData<ValueType, int64>,
       public EnableAbsoluteComputation<remove_complex<Diagonal<ValueType>>> {
     friend class EnablePolymorphicObject<Diagonal, LinOp>;
-    friend class EnableCreateMethod<Diagonal>;
     friend class Csr<ValueType, int32>;
     friend class Csr<ValueType, int64>;
     friend class Diagonal<to_complex<ValueType>>;
@@ -192,6 +162,50 @@ public:
 
     void write(mat_data32& data) const override;
 
+    /**
+     * Creates an Diagonal matrix of the specified size.
+     *
+     * @param exec  Executor associated to the matrix
+     * @param size  size of the matrix
+     *
+     * @return A smart pointer to the newly created matrix.
+     */
+    static std::unique_ptr<Diagonal> create(
+        std::shared_ptr<const Executor> exec, size_type size = 0);
+
+    /**
+     * Creates a Diagonal matrix from an already allocated (and initialized)
+     * array.
+     *
+     * @param exec  Executor associated to the matrix
+     * @param size  size of the matrix
+     * @param values  array of matrix values
+     *
+     * @note If `values` is not an rvalue, not an array of ValueType, or is on
+     *       the wrong executor, an internal copy will be created, and the
+     *       original array data will not be used in the matrix.
+     *
+     * @return A smart pointer to the newly created matrix.
+     */
+    static std::unique_ptr<Diagonal> create(
+        std::shared_ptr<const Executor> exec, const size_type size,
+        array<value_type> values);
+
+    /**
+     * @copydoc std::unique_ptr<Diagonal> create(std::shared_ptr<const
+     * Executor>, const size_type, array<value_type>)
+     */
+    template <typename InputValueType>
+    GKO_DEPRECATED(
+        "explicitly construct the gko::array argument instead of passing an"
+        "initializer list")
+    static std::unique_ptr<Diagonal> create(
+        std::shared_ptr<const Executor> exec, const size_type size,
+        std::initializer_list<InputValueType> values)
+    {
+        return create(exec, size, array<value_type>{exec, std::move(values)});
+    }
+
     /**
      * Creates a constant (immutable) Diagonal matrix from a constant array.
      *
@@ -204,56 +218,13 @@ public:
      */
     static std::unique_ptr<const Diagonal> create_const(
         std::shared_ptr<const Executor> exec, size_type size,
-        gko::detail::const_array_view<ValueType>&& values)
-    {
-        // cast const-ness away, but return a const object afterwards,
-        // so we can ensure that no modifications take place.
-        return std::unique_ptr<const Diagonal>(new Diagonal{
-            exec, size, gko::detail::array_const_cast(std::move(values))});
-    }
+        gko::detail::const_array_view<ValueType>&& values);
 
 protected:
-    /**
-     * Creates an empty Diagonal matrix.
-     *
-     * @param exec  Executor associated to the matrix
-     */
-    explicit Diagonal(std::shared_ptr<const Executor> exec)
-        : Diagonal(std::move(exec), size_type{})
-    {}
+    Diagonal(std::shared_ptr<const Executor> exec, size_type size = 0);
 
-    /**
-     * Creates an Diagonal matrix of the specified size.
-     *
-     * @param exec  Executor associated to the matrix
-     * @param size  size of the matrix
-     */
-    Diagonal(std::shared_ptr<const Executor> exec, size_type size)
-        : EnableLinOp<Diagonal>(exec, dim<2>{size}), values_(exec, size)
-    {}
-
-    /**
-     * Creates a Diagonal matrix from an already allocated (and initialized)
-     * array.
-     *
-     * @tparam ValuesArray  type of array of values
-     *
-     * @param exec  Executor associated to the matrix
-     * @param size  size of the matrix
-     * @param values  array of matrix values
-     *
-     * @note If `values` is not an rvalue, not an array of ValueType, or is on
-     *       the wrong executor, an internal copy will be created, and the
-     *       original array data will not be used in the matrix.
-     */
-    template <typename ValuesArray>
     Diagonal(std::shared_ptr<const Executor> exec, const size_type size,
-             ValuesArray&& values)
-        : EnableLinOp<Diagonal>(exec, dim<2>(size)),
-          values_{exec, std::forward<ValuesArray>(values)}
-    {
-        GKO_ENSURE_IN_BOUNDS(size - 1, values_.get_num_elems());
-    }
+             array<value_type> values);
 
     void apply_impl(const LinOp* b, LinOp* x) const override;
 
