@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #ifndef GKO_PUBLIC_CORE_PRECONDITIONER_IC_HPP_
 #define GKO_PUBLIC_CORE_PRECONDITIONER_IC_HPP_
@@ -45,8 +17,14 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/base/lin_op.hpp>
 #include <ginkgo/core/base/precision_dispatch.hpp>
 #include <ginkgo/core/base/std_extensions.hpp>
+#include <ginkgo/core/config/config.hpp>
+#include <ginkgo/core/config/registry.hpp>
 #include <ginkgo/core/factorization/par_ic.hpp>
 #include <ginkgo/core/matrix/dense.hpp>
+#include <ginkgo/core/preconditioner/isai.hpp>
+#include <ginkgo/core/preconditioner/utils.hpp>
+#include <ginkgo/core/solver/gmres.hpp>
+#include <ginkgo/core/solver/ir.hpp>
 #include <ginkgo/core/solver/solver_traits.hpp>
 #include <ginkgo/core/solver/triangular.hpp>
 #include <ginkgo/core/stop/combined.hpp>
@@ -56,8 +34,38 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 namespace gko {
 namespace preconditioner {
+namespace detail {
 
 
+template <typename Type>
+constexpr bool support_ic_parse =
+    is_instantiation_of<Type, solver::LowerTrs>::value ||
+    is_instantiation_of<Type, solver::Ir>::value ||
+    is_instantiation_of<Type, solver::Gmres>::value ||
+    is_instantiation_of<Type, preconditioner::LowerIsai>::value;
+
+
+template <
+    typename Ic,
+    std::enable_if_t<!support_ic_parse<typename Ic::l_solver_type>>* = nullptr>
+typename Ic::parameters_type ic_parse(
+    const config::pnode& config, const config::registry& context,
+    const config::type_descriptor& td_for_child)
+{
+    GKO_INVALID_STATE(
+        "preconditioner::Ic only supports limited type for parse.");
+}
+
+template <
+    typename Ic,
+    std::enable_if_t<support_ic_parse<typename Ic::l_solver_type>>* = nullptr>
+typename Ic::parameters_type ic_parse(
+    const config::pnode& config, const config::registry& context,
+    const config::type_descriptor& td_for_child);
+
+
+}  // namespace detail
+
 /**
  * The Incomplete Cholesky (IC) preconditioner solves the equation $LL^H*x = b$
  * for a given lower triangular matrix L and the right hand side b (can contain
@@ -67,7 +75,7 @@ namespace preconditioner {
  * is a direct triangular solvers. The solver for L^H is the
  * conjugate-transposed solver for L, ensuring that the preconditioner is
  * symmetric and positive-definite. For this L solver, a factory can be provided
- * (using `with_l_solver_factory`) to have more control over their behavior. In
+ * (using `with_l_solver`) to have more control over their behavior. In
  * particular, it is possible to use an iterative method for solving the
  * triangular systems. The default parameters for an iterative triangluar solver
  * are:
@@ -119,24 +127,99 @@ public:
     using index_type = IndexType;
     using transposed_type = Ic<LSolverType, IndexType>;
 
-    GKO_CREATE_FACTORY_PARAMETERS(parameters, Factory)
-    {
+    class Factory;
+
+    struct parameters_type
+        : public enable_parameters_type<parameters_type, Factory> {
         /**
          * Factory for the L solver
          */
-        std::shared_ptr<typename l_solver_type::Factory>
-            GKO_FACTORY_PARAMETER_SCALAR(l_solver_factory, nullptr);
+        std::shared_ptr<const typename l_solver_type::Factory>
+            l_solver_factory{};
 
         /**
          * Factory for the factorization
          */
-        std::shared_ptr<LinOpFactory> GKO_FACTORY_PARAMETER_SCALAR(
-            factorization_factory, nullptr);
+        std::shared_ptr<const LinOpFactory> factorization_factory{};
+
+        GKO_DEPRECATED("use with_l_solver instead")
+        parameters_type& with_l_solver_factory(
+            deferred_factory_parameter<const typename l_solver_type::Factory>
+                solver)
+        {
+            return with_l_solver(std::move(solver));
+        }
+
+        parameters_type& with_l_solver(
+            deferred_factory_parameter<const typename l_solver_type::Factory>
+                solver)
+        {
+            this->l_solver_generator = std::move(solver);
+            this->deferred_factories["l_solver"] = [](const auto& exec,
+                                                      auto& params) {
+                if (!params.l_solver_generator.is_empty()) {
+                    params.l_solver_factory =
+                        params.l_solver_generator.on(exec);
+                }
+            };
+            return *this;
+        }
+
+        GKO_DEPRECATED("use with_factorization instead")
+        parameters_type& with_factorization_factory(
+            deferred_factory_parameter<const LinOpFactory> factorization)
+        {
+            return with_factorization(std::move(factorization));
+        }
+
+        parameters_type& with_factorization(
+            deferred_factory_parameter<const LinOpFactory> factorization)
+        {
+            this->factorization_generator = std::move(factorization);
+            this->deferred_factories["factorization"] = [](const auto& exec,
+                                                           auto& params) {
+                if (!params.factorization_generator.is_empty()) {
+                    params.factorization_factory =
+                        params.factorization_generator.on(exec);
+                }
+            };
+            return *this;
+        }
+
+    private:
+        deferred_factory_parameter<const typename l_solver_type::Factory>
+            l_solver_generator;
+
+        deferred_factory_parameter<const LinOpFactory> factorization_generator;
     };
 
     GKO_ENABLE_LIN_OP_FACTORY(Ic, parameters, Factory);
     GKO_ENABLE_BUILD_METHOD(Factory);
 
+    /**
+     * Create the parameters from the property_tree.
+     * Because this is directly tied to the specific type, the value/index type
+     * settings within config are ignored and type_descriptor is only used
+     * for children configs.
+     *
+     * @param config  the property tree for setting
+     * @param context  the registry
+     * @param td_for_child  the type descriptor for children configs. The
+     *                      default uses the value/index type of this class.
+     *
+     * @return parameters
+     *
+     * @note only support the following for l_solver:
+     *       Ir, Gmres, LowerTrs, and LowerIsai
+     */
+    static parameters_type parse(
+        const config::pnode& config, const config::registry& context,
+        const config::type_descriptor& td_for_child =
+            config::make_type_descriptor<value_type, index_type>())
+    {
+        return detail::ic_parse<Ic>(config, context, td_for_child);
+    }
+
     /**
      * Returns the solver which is used for the provided L matrix.
      *
@@ -365,12 +448,10 @@ protected:
             static_cast<unsigned int>(mtx->get_size()[0])};
 
         return SolverType::build()
-            .with_criteria(gko::stop::Iteration::build()
-                               .with_max_iters(default_max_iters)
-                               .on(exec),
-                           gko::stop::ResidualNorm<value_type>::build()
-                               .with_reduction_factor(default_reduce_residual)
-                               .on(exec))
+            .with_criteria(
+                gko::stop::Iteration::build().with_max_iters(default_max_iters),
+                gko::stop::ResidualNorm<value_type>::build()
+                    .with_reduction_factor(default_reduce_residual))
             .on(exec)
             ->generate(mtx);
     }
