@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/distributed/partition_kernels.hpp"
 
@@ -44,7 +16,7 @@ void count_ranges(std::shared_ptr<const DefaultExecutor> exec,
 {
     num_ranges = 0;
     comm_index_type prev_part{-1};
-    for (size_type i = 0; i < mapping.get_num_elems(); i++) {
+    for (size_type i = 0; i < mapping.get_size(); i++) {
         auto cur_part = mapping.get_const_data()[i];
         num_ranges += cur_part != prev_part;
         prev_part = cur_part;
@@ -59,9 +31,9 @@ void build_from_contiguous(std::shared_ptr<const DefaultExecutor> exec,
                            GlobalIndexType* range_bounds,
                            comm_index_type* part_ids)
 {
-    bool uses_mapping = part_id_mapping.get_num_elems() > 0;
+    bool uses_mapping = part_id_mapping.get_size() > 0;
     range_bounds[0] = 0;
-    for (comm_index_type i = 0; i < ranges.get_num_elems() - 1; i++) {
+    for (comm_index_type i = 0; i < ranges.get_size() - 1; i++) {
         auto end = ranges.get_const_data()[i + 1];
         range_bounds[i + 1] = end;
         part_ids[i] = uses_mapping ? part_id_mapping.get_const_data()[i] : i;
@@ -79,7 +51,7 @@ void build_from_mapping(std::shared_ptr<const DefaultExecutor> exec,
 {
     size_type range_idx{};
     comm_index_type range_part{-1};
-    for (size_type i = 0; i < mapping.get_num_elems(); i++) {
+    for (size_type i = 0; i < mapping.get_size(); i++) {
         auto cur_part = mapping.get_const_data()[i];
         if (cur_part != range_part) {
             range_bounds[range_idx] = i;
@@ -88,8 +60,7 @@ void build_from_mapping(std::shared_ptr<const DefaultExecutor> exec,
             range_part = cur_part;
         }
     }
-    range_bounds[range_idx] =
-        static_cast<GlobalIndexType>(mapping.get_num_elems());
+    range_bounds[range_idx] = static_cast<GlobalIndexType>(mapping.get_size());
 }
 
 GKO_INSTANTIATE_FOR_EACH_INDEX_TYPE(GKO_PARTITION_BUILD_FROM_MAPPING);
