@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/matrix/hybrid_kernels.hpp"
 
@@ -61,7 +33,7 @@ void compute_coo_row_ptrs(std::shared_ptr<const DefaultExecutor> exec,
                           const array<size_type>& row_nnz, size_type ell_lim,
                           int64* coo_row_ptrs)
 {
-    for (size_type row = 0; row < row_nnz.get_num_elems(); row++) {
+    for (size_type row = 0; row < row_nnz.get_size(); row++) {
         if (row_nnz.get_const_data()[row] <= ell_lim) {
             coo_row_ptrs[row] = 0;
         } else {
@@ -69,14 +41,14 @@ void compute_coo_row_ptrs(std::shared_ptr<const DefaultExecutor> exec,
         }
     }
     components::prefix_sum_nonnegative(exec, coo_row_ptrs,
-                                       row_nnz.get_num_elems() + 1);
+                                       row_nnz.get_size() + 1);
 }
 
 
 void compute_row_nnz(std::shared_ptr<const DefaultExecutor> exec,
                      const array<int64>& row_ptrs, size_type* row_nnzs)
 {
-    for (size_type i = 0; i < row_ptrs.get_num_elems() - 1; i++) {
+    for (size_type i = 0; i < row_ptrs.get_size() - 1; i++) {
         row_nnzs[i] =
             row_ptrs.get_const_data()[i + 1] - row_ptrs.get_const_data()[i];
     }
