@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include "core/base/utils.hpp"
 
@@ -66,13 +38,16 @@ protected:
     ConvertToWithSorting()
         : ref{gko::ReferenceExecutor::create()},
           mtx{gko::initialize<Dense>({{1, 2, 3}, {6, 0, 7}, {-1, 8, 0}}, ref)},
-          unsorted_coo{Coo::create(ref, gko::dim<2>{3, 3},
-                                   I<value_type>{1, 3, 2, 7, 6, -1, 8},
-                                   I<index_type>{0, 2, 1, 2, 0, 0, 1},
-                                   I<index_type>{0, 0, 0, 1, 1, 2, 2})},
-          unsorted_csr{Csr::create(
-              ref, gko::dim<2>{3, 3}, I<value_type>{1, 3, 2, 7, 6, -1, 8},
-              I<index_type>{0, 2, 1, 2, 0, 0, 1}, I<index_type>{0, 3, 5, 7})}
+          unsorted_coo{
+              Coo::create(ref, gko::dim<2>{3, 3},
+                          gko::array<value_type>{ref, {1, 3, 2, 7, 6, -1, 8}},
+                          gko::array<index_type>{ref, {0, 2, 1, 2, 0, 0, 1}},
+                          gko::array<index_type>{ref, {0, 0, 0, 1, 1, 2, 2}})},
+          unsorted_csr{
+              Csr::create(ref, gko::dim<2>{3, 3},
+                          gko::array<value_type>{ref, {1, 3, 2, 7, 6, -1, 8}},
+                          gko::array<index_type>{ref, {0, 2, 1, 2, 0, 0, 1}},
+                          gko::array<index_type>{ref, {0, 3, 5, 7}})}
 
     {}
 
