@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/distributed/partition.hpp>
 
@@ -96,7 +68,7 @@ TYPED_TEST(Partition, BuildsFromMapping)
     auto partition =
         part_type::build_from_mapping(this->ref, mapping, num_parts);
 
-    EXPECT_EQ(partition->get_size(), mapping.get_num_elems());
+    EXPECT_EQ(partition->get_size(), mapping.get_size());
     EXPECT_EQ(partition->get_num_ranges(), num_ranges);
     EXPECT_EQ(partition->get_num_parts(), num_parts);
     EXPECT_EQ(partition->get_num_empty_parts(), 0);
@@ -121,7 +93,7 @@ TYPED_TEST(Partition, BuildsFromMappingWithEmptyParts)
     auto partition =
         part_type::build_from_mapping(this->ref, mapping, num_parts);
 
-    EXPECT_EQ(partition->get_size(), mapping.get_num_elems());
+    EXPECT_EQ(partition->get_size(), mapping.get_size());
     EXPECT_EQ(partition->get_num_ranges(), num_ranges);
     EXPECT_EQ(partition->get_num_parts(), num_parts);
     EXPECT_EQ(partition->get_num_empty_parts(), 2);
@@ -143,10 +115,9 @@ TYPED_TEST(Partition, BuildsFromRanges)
 
     auto partition = part_type::build_from_contiguous(this->ref, ranges);
 
-    EXPECT_EQ(partition->get_size(),
-              ranges.get_data()[ranges.get_num_elems() - 1]);
-    EXPECT_EQ(partition->get_num_ranges(), ranges.get_num_elems() - 1);
-    EXPECT_EQ(partition->get_num_parts(), ranges.get_num_elems() - 1);
+    EXPECT_EQ(partition->get_size(), ranges.get_data()[ranges.get_size() - 1]);
+    EXPECT_EQ(partition->get_num_ranges(), ranges.get_size() - 1);
+    EXPECT_EQ(partition->get_num_parts(), ranges.get_size() - 1);
     EXPECT_EQ(partition->get_num_empty_parts(), 1);
     assert_equal_data(partition->get_range_bounds(), {0, 5, 5, 7, 9, 10});
     assert_equal_data(partition->get_part_ids(), {0, 1, 2, 3, 4});
@@ -181,10 +152,9 @@ TYPED_TEST(Partition, BuildsFromRangesWithPartIds)
     auto partition =
         part_type::build_from_contiguous(this->ref, ranges, part_id);
 
-    EXPECT_EQ(partition->get_size(),
-              ranges.get_data()[ranges.get_num_elems() - 1]);
-    EXPECT_EQ(partition->get_num_ranges(), ranges.get_num_elems() - 1);
-    EXPECT_EQ(partition->get_num_parts(), ranges.get_num_elems() - 1);
+    EXPECT_EQ(partition->get_size(), ranges.get_data()[ranges.get_size() - 1]);
+    EXPECT_EQ(partition->get_num_ranges(), ranges.get_size() - 1);
+    EXPECT_EQ(partition->get_num_parts(), ranges.get_size() - 1);
     EXPECT_EQ(partition->get_num_empty_parts(), 1);
     assert_equal_data(partition->get_range_bounds(), {0, 5, 5, 7, 9, 10});
     assert_equal_data(partition->get_part_ids(), {0, 4, 3, 1, 2});
@@ -245,6 +215,23 @@ TYPED_TEST(Partition, BuildsFromGlobalSizeWithEmptyParts)
 }
 
 
+TYPED_TEST(Partition, BuildsFromGlobalSizeWithZeroParts)
+{
+    using part_type = typename TestFixture::part_type;
+
+    auto partition = part_type::build_from_global_size_uniform(this->ref, 0, 3);
+
+    EXPECT_EQ(partition->get_size(), 0);
+    EXPECT_EQ(partition->get_num_ranges(), 0);
+    EXPECT_EQ(partition->get_num_parts(), 0);
+    EXPECT_EQ(partition->get_num_empty_parts(), 0);
+    assert_equal_data(partition->get_range_bounds(), {0});
+    ASSERT_EQ(partition->get_part_ids(), nullptr);
+    ASSERT_EQ(partition->get_range_starting_indices(), nullptr);
+    ASSERT_EQ(partition->get_part_sizes(), nullptr);
+}
+
+
 TYPED_TEST(Partition, IsConnected)
 {
     using part_type = typename TestFixture::part_type;
