@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/matrix/csr.hpp>
 
@@ -2446,7 +2418,7 @@ TYPED_TEST(Csr, CanGetSubmatrix2)
 }
 
 
-TYPED_TEST(Csr, CanGetSubmatrixWithindex_set)
+TYPED_TEST(Csr, CanGetSubmatrixWithIndexSet)
 {
     using Vec = typename TestFixture::Vec;
     using Mtx = typename TestFixture::Mtx;
@@ -2604,7 +2576,7 @@ protected:
         // row 3: very sparse
         data.nonzeros.emplace_back(3, 0, 1.0);
         data.nonzeros.emplace_back(3, 64, 1.0);
-        data.ensure_row_major_order();
+        data.sort_row_major();
         // 1000 as min-sentinel
         full_sizes = {0, 0, 1000, 1000, 0, 0};
         bitmap_sizes = {0, 6, 4, 6, 2, 2};
@@ -2626,6 +2598,7 @@ TYPED_TEST_SUITE(CsrLookup, gko::test::ValueIndexTypes,
 TYPED_TEST(CsrLookup, GeneratesLookupDataOffsets)
 {
     using IndexType = typename TestFixture::index_type;
+    using gko::matrix::csr::csr_lookup_allowed;
     using gko::matrix::csr::sparsity_type;
     const auto num_rows = this->mtx->get_size()[0];
     gko::array<IndexType> storage_offset_array(this->exec, num_rows + 1);
@@ -2636,19 +2609,19 @@ TYPED_TEST(CsrLookup, GeneratesLookupDataOffsets)
     for (auto allowed :
          {sparsity_type::full | sparsity_type::bitmap | sparsity_type::hash,
           sparsity_type::bitmap | sparsity_type::hash,
-          sparsity_type::full | sparsity_type::hash, sparsity_type::hash}) {
+          sparsity_type::full | sparsity_type::hash, sparsity_type::hash,
+          sparsity_type::none}) {
         gko::kernels::reference::csr::build_lookup_offsets(
             this->exec, row_ptrs, col_idxs, num_rows, allowed, storage_offsets);
-        bool allow_full =
-            gko::matrix::csr::csr_lookup_allowed(allowed, sparsity_type::full);
-        bool allow_bitmap = gko::matrix::csr::csr_lookup_allowed(
-            allowed, sparsity_type::bitmap);
+        bool allow_full = csr_lookup_allowed(allowed, sparsity_type::full);
+        bool allow_bitmap = csr_lookup_allowed(allowed, sparsity_type::bitmap);
+        bool allow_hash = csr_lookup_allowed(allowed, sparsity_type::hash);
 
         for (gko::size_type row = 0; row < num_rows; row++) {
-            const auto expected_size =
-                std::min(allow_full ? this->full_sizes[row] : 1000,
-                         std::min(allow_bitmap ? this->bitmap_sizes[row] : 1000,
-                                  this->hash_sizes[row]));
+            const auto expected_size = std::min(
+                allow_full ? this->full_sizes[row] : 1000,
+                std::min(allow_bitmap ? this->bitmap_sizes[row] : 1000,
+                         allow_hash ? this->hash_sizes[row] : IndexType{}));
             const auto size = storage_offsets[row + 1] - storage_offsets[row];
 
             ASSERT_EQ(size, expected_size);
@@ -2672,16 +2645,21 @@ TYPED_TEST(CsrLookup, GeneratesLookupData)
     for (auto allowed :
          {sparsity_type::full | sparsity_type::bitmap | sparsity_type::hash,
           sparsity_type::bitmap | sparsity_type::hash,
-          sparsity_type::full | sparsity_type::hash, sparsity_type::hash}) {
+          sparsity_type::full | sparsity_type::hash, sparsity_type::hash,
+          sparsity_type::none}) {
         gko::kernels::reference::csr::build_lookup_offsets(
             this->exec, row_ptrs, col_idxs, num_rows, allowed, storage_offsets);
         gko::array<gko::int32> storage_array(this->exec,
                                              storage_offsets[num_rows]);
         const auto storage = storage_array.get_data();
+        const auto hash_equivalent =
+            csr_lookup_allowed(allowed, sparsity_type::hash)
+                ? sparsity_type::hash
+                : sparsity_type::none;
         const auto bitmap_equivalent =
             csr_lookup_allowed(allowed, sparsity_type::bitmap)
                 ? sparsity_type::bitmap
-                : sparsity_type::hash;
+                : hash_equivalent;
         const auto full_equivalent =
             csr_lookup_allowed(allowed, sparsity_type::full)
                 ? sparsity_type::full
@@ -2715,7 +2693,7 @@ TYPED_TEST(CsrLookup, GeneratesLookupData)
         ASSERT_EQ(row_descs[0] & 0xF, static_cast<int>(full_equivalent));
         ASSERT_EQ(row_descs[1] & 0xF, static_cast<int>(full_equivalent));
         ASSERT_EQ(row_descs[2] & 0xF, static_cast<int>(bitmap_equivalent));
-        ASSERT_EQ(row_descs[3] & 0xF, static_cast<int>(sparsity_type::hash));
+        ASSERT_EQ(row_descs[3] & 0xF, static_cast<int>(hash_equivalent));
         ASSERT_EQ(row_descs[4] & 0xF, static_cast<int>(full_equivalent));
         ASSERT_EQ(row_descs[5] & 0xF, static_cast<int>(full_equivalent));
     }
