@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/solver/gcr.hpp>
 
@@ -125,8 +97,7 @@ protected:
         stopped.converge(1, true);
         non_stopped.reset();
         small_stop = gko::array<gko::stopping_status>(exec, small_size[1]);
-        std::fill_n(small_stop.get_data(), small_stop.get_num_elems(),
-                    non_stopped);
+        std::fill_n(small_stop.get_data(), small_stop.get_size(), non_stopped);
         small_final_iter_nums = gko::array<gko::size_type>(exec, small_size[1]);
     }
 
@@ -161,7 +132,7 @@ TYPED_TEST(Gcr, KernelInitialize)
     using T = typename TestFixture::value_type;
     const T nan = std::numeric_limits<gko::remove_complex<T>>::quiet_NaN();
     this->small_residual->fill(nan);
-    std::fill_n(this->small_stop.get_data(), this->small_stop.get_num_elems(),
+    std::fill_n(this->small_stop.get_data(), this->small_stop.get_size(),
                 this->stopped);
 
     gko::kernels::reference::gcr::initialize(this->exec, this->small_b.get(),
@@ -169,7 +140,7 @@ TYPED_TEST(Gcr, KernelInitialize)
                                              this->small_stop.get_data());
 
     GKO_ASSERT_MTX_NEAR(this->small_residual, this->small_b, 0);
-    for (int i = 0; i < this->small_stop.get_num_elems(); ++i) {
+    for (int i = 0; i < this->small_stop.get_size(); ++i) {
         ASSERT_EQ(this->small_stop.get_data()[i], this->non_stopped);
     }
 }
@@ -186,7 +157,7 @@ TYPED_TEST(Gcr, KernelRestart)
     this->small_krylov_bases_p->fill(nan);
     this->small_mapped_krylov_bases_Ap->fill(nan);
     std::fill_n(this->small_final_iter_nums.get_data(),
-                this->small_final_iter_nums.get_num_elems(), 999);
+                this->small_final_iter_nums.get_size(), 999);
     auto expected_p_bases = gko::clone(this->exec, this->small_krylov_bases_p);
     auto expected_Ap_bases =
         gko::clone(this->exec, this->small_mapped_krylov_bases_Ap);
@@ -207,9 +178,9 @@ TYPED_TEST(Gcr, KernelRestart)
         this->small_mapped_krylov_bases_Ap.get(),
         this->small_final_iter_nums.get_data());
 
-    ASSERT_EQ(this->small_final_iter_nums.get_num_elems(),
+    ASSERT_EQ(this->small_final_iter_nums.get_size(),
               this->small_residual->get_size()[1]);
-    for (int i = 0; i < this->small_final_iter_nums.get_num_elems(); ++i) {
+    for (int i = 0; i < this->small_final_iter_nums.get_size(); ++i) {
         ASSERT_EQ(this->small_final_iter_nums.get_const_data()[i], 0);
     }
     GKO_ASSERT_MTX_NEAR(this->small_krylov_bases_p, this->small_residual,
