@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <ginkgo/core/solver/gmres.hpp>
 
@@ -140,8 +112,7 @@ protected:
         stopped.converge(1, true);
         non_stopped.reset();
         small_stop = gko::array<gko::stopping_status>(exec, small_size[1]);
-        std::fill_n(small_stop.get_data(), small_stop.get_num_elems(),
-                    non_stopped);
+        std::fill_n(small_stop.get_data(), small_stop.get_size(), non_stopped);
         small_final_iter_nums = gko::array<gko::size_type>(exec, small_size[1]);
     }
 
@@ -180,7 +151,7 @@ TYPED_TEST(Gmres, KernelInitialize)
     this->small_residual->fill(nan);
     this->small_givens_sin->fill(nan);
     this->small_givens_cos->fill(nan);
-    std::fill_n(this->small_stop.get_data(), this->small_stop.get_num_elems(),
+    std::fill_n(this->small_stop.get_data(), this->small_stop.get_size(),
                 this->stopped);
     auto expected_sin_cos =
         Mtx::create(this->exec, this->small_givens_sin->get_size());
@@ -194,7 +165,7 @@ TYPED_TEST(Gmres, KernelInitialize)
     GKO_ASSERT_MTX_NEAR(this->small_residual, this->small_b, 0);
     GKO_ASSERT_MTX_NEAR(this->small_givens_sin, expected_sin_cos, 0);
     GKO_ASSERT_MTX_NEAR(this->small_givens_cos, expected_sin_cos, 0);
-    for (int i = 0; i < this->small_stop.get_num_elems(); ++i) {
+    for (int i = 0; i < this->small_stop.get_size(); ++i) {
         ASSERT_EQ(this->small_stop.get_data()[i], this->non_stopped);
     }
 }
@@ -211,7 +182,7 @@ TYPED_TEST(Gmres, KernelRestart)
     this->small_residual_norm_collection->fill(nan);
     this->small_krylov_bases->fill(9999);
     std::fill_n(this->small_final_iter_nums.get_data(),
-                this->small_final_iter_nums.get_num_elems(), 999);
+                this->small_final_iter_nums.get_size(), 999);
     auto expected_krylov = gko::clone(this->exec, this->small_krylov_bases);
     const auto small_size = this->small_residual->get_size();
     for (int i = 0; i < small_size[0]; ++i) {
@@ -229,9 +200,9 @@ TYPED_TEST(Gmres, KernelRestart)
         this->small_residual_norm_collection.get(),
         this->small_krylov_bases.get(), this->small_final_iter_nums.get_data());
 
-    ASSERT_EQ(this->small_final_iter_nums.get_num_elems(),
+    ASSERT_EQ(this->small_final_iter_nums.get_size(),
               this->small_residual_norm_collection->get_size()[1]);
-    for (int i = 0; i < this->small_final_iter_nums.get_num_elems(); ++i) {
+    for (int i = 0; i < this->small_final_iter_nums.get_size(); ++i) {
         ASSERT_EQ(this->small_final_iter_nums.get_const_data()[i], 0);
         ASSERT_EQ(this->small_residual_norm_collection->get_const_values()[i],
                   this->small_residual_norm->get_const_values()[i]);
