@@ -1,34 +1,6 @@
-/*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2023, the Ginkgo authors
-All rights reserved.
-
-Redistribution and use in source and binary forms, with or without
-modification, are permitted provided that the following conditions
-are met:
-
-1. Redistributions of source code must retain the above copyright
-notice, this list of conditions and the following disclaimer.
-
-2. Redistributions in binary form must reproduce the above copyright
-notice, this list of conditions and the following disclaimer in the
-documentation and/or other materials provided with the distribution.
-
-3. Neither the name of the copyright holder nor the names of its
-contributors may be used to endorse or promote products derived from
-this software without specific prior written permission.
-
-THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
-IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
-TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-******************************<GINKGO LICENSE>*******************************/
+// SPDX-FileCopyrightText: 2017 - 2024 The Ginkgo authors
+//
+// SPDX-License-Identifier: BSD-3-Clause
 
 #include <random>
 
@@ -177,6 +149,9 @@ TYPED_TEST(Fft, ApplyStrided1DInverseIsEqualToReference)
 
 TYPED_TEST(Fft, Apply2DIsEqualToReference)
 {
+#if defined(GKO_COMPILING_HIP) && GINKGO_HIP_PLATFORM_HCC
+    GTEST_SKIP() << "rocFFT 5.1 has a bug related to 2D FFT";
+#endif
     using T = typename TestFixture::value_type;
 
     this->fft2->apply(this->data, this->out);
@@ -188,6 +163,9 @@ TYPED_TEST(Fft, Apply2DIsEqualToReference)
 
 TYPED_TEST(Fft, ApplyStrided2DIsEqualToReference)
 {
+#if defined(GKO_COMPILING_HIP) && GINKGO_HIP_PLATFORM_HCC
+    GTEST_SKIP() << "rocFFT 5.1 has a bug related to 2D FFT";
+#endif
     using T = typename TestFixture::value_type;
 
     this->fft2->apply(this->data_strided, this->out_strided);
@@ -199,6 +177,9 @@ TYPED_TEST(Fft, ApplyStrided2DIsEqualToReference)
 
 TYPED_TEST(Fft, Apply2DInverseIsEqualToReference)
 {
+#if defined(GKO_COMPILING_HIP) && GINKGO_HIP_PLATFORM_HCC
+    GTEST_SKIP() << "rocFFT 5.1 has a bug related to 2D FFT";
+#endif
     using T = typename TestFixture::value_type;
 
     this->ifft2->apply(this->data, this->out);
@@ -210,6 +191,9 @@ TYPED_TEST(Fft, Apply2DInverseIsEqualToReference)
 
 TYPED_TEST(Fft, ApplyStrided2DInverseIsEqualToReference)
 {
+#if defined(GKO_COMPILING_HIP) && GINKGO_HIP_PLATFORM_HCC
+    GTEST_SKIP() << "rocFFT 5.1 has a bug related to 2D FFT";
+#endif
     using T = typename TestFixture::value_type;
 
     this->ifft2->apply(this->data_strided, this->out_strided);
